@@ -8,14 +8,20 @@ class Pdb:  # TODO optionally save as gro? & think of trajectories
                 'MET': 'M', 'ASN': 'N', 'PRO': 'P', 'GLN': 'Q', 'ARG': 'R', 'SER': 'S', 'THR': 'T', 'VAL': 'V',
                 'TRP': 'W', 'TYR': 'Y', "GLUP": "E", "ASPP": "D"}
 
-    def __init__(self, filename=None, top=None, altloc='A', **kwargs):
+    nucl_map = {'DA': "A", 'DG': "G", 'DC': "C", 'DT': "T", 'DA5': "A", 'DG5': "G", 'DC5': "C", 'DT5': "T",
+                'DA3': "A", 'DG3': "G", 'DC3': "C", 'DT3': "T", 'RA': "A", 'RG': "G", 'RC': "C", 'RU': "U",
+                'RA5': "A", 'RG5': "G", 'RC5': "C", 'RU5': "U", 'RA3': "A", 'RG3': "G", 'RC3': "C", 'RU3': "U",
+                'A': "A", 'G': "G", 'C': "C", 'U': "U", 'A5': "A", 'G5': "G", 'C5': "C", 'U5': "U",
+                'A3': "A", 'G3': "G", 'C3': "C", 'U3': "U"}
+
+    def __init__(self, filename=None, top=None, altloc='A', qt=False, **kwargs):
         self.fname = filename
         if self.fname:
             if self.fname.endswith('gro'):
                 self.atoms, self.box, self.remarks = self._parse_contents_gro([line.rstrip()
                                                                                for line in open(self.fname)])
             else:
-                self.atoms, self.box, self.remarks = self._parse_contents([line.strip() for line in open(self.fname)])
+                self.atoms, self.box, self.remarks = self._parse_contents([line.strip() for line in open(self.fname)], qt)
         else:
             self.atoms, self.box, self.remarks = [], 3 * [100] + 3 * [90], []
         self.top = top if not isinstance(top, str) else gml.Top(top, **kwargs)
@@ -70,7 +76,9 @@ class Pdb:  # TODO optionally save as gro? & think of trajectories
         except:
             ftype = orig_pdb.sfname[-3:]
         if ftype == 'pdb':
-            new_pdb.atoms = cls._parse_contents([line.strip() for line in text.split('\n')])
+            new_pdb.atoms = cls._parse_contents([line.strip() for line in text.split('\n')], False)
+        elif ftype == 'bqt':
+            new_pdb.atoms = cls._parse_contents([line.strip() for line in text.split('\n')], True)
         elif ftype == 'gro':
             new_pdb.atoms = cls._parse_contents_gro([line.strip() for line in text.split('\n')])
         new_pdb.box = orig_pdb.box
@@ -107,11 +115,7 @@ class Pdb:  # TODO optionally save as gro? & think of trajectories
     def print_nucleic_sequence(self):
         mapping = defaultdict(lambda: 'X')
         sequences = []
-        mapping.update({'DA': "A", 'DG': "G", 'DC': "C", 'DT': "T", 'DA5': "A", 'DG5': "G", 'DC5': "C", 'DT5': "T",
-                        'DA3': "A", 'DG3': "G", 'DC3': "C", 'DT3': "T", 'RA': "A", 'RG': "G", 'RC': "C", 'RU': "U",
-                        'RA5': "A", 'RG5': "G", 'RC5': "C", 'RU5': "U", 'RA3': "A", 'RG3': "G", 'RC3': "C", 'RU3': "U",
-                        'A': "A", 'G': "G", 'C': "C", 'U': "U", 'A5': "A", 'G5': "G", 'C5': "C", 'U5': "U",
-                        'A3': "A", 'G3': "G", 'C3': "C", 'U3': "U"})
+        mapping.update(Pdb.nucl_map)
         chains = list({a.chain.strip() for a in self.atoms if a.atomname == "O4'"})
         if not chains:
             print("No chains in the molecule, run Pdb.add_chains() first")
@@ -123,11 +127,6 @@ class Pdb:  # TODO optionally save as gro? & think of trajectories
         return sequences
 
     def find_missing(self):
-        map_nuc = {'DA': "A", 'DG': "G", 'DC': "C", 'DT': "T", 'DA5': "A", 'DG5': "G", 'DC5': "C", 'DT5': "T",
-                   'DA3': "A", 'DG3': "G", 'DC3': "C", 'DT3': "T", 'RA': "A", 'RG': "G", 'RC': "C", 'RU': "U",
-                   'RA5': "A", 'RG5': "G", 'RC5': "C", 'RU5': "U", 'RA3': "A", 'RG3': "G", 'RC3': "C", 'RU3': "U",
-                   'A': "A", 'G': "G", 'C': "C", 'U': "U", 'A5': "A", 'G5': "G", 'C5': "C", 'U5': "U",
-                   'A3': "A", 'G3': "G", 'C3': "C", 'U3': "U"}
         pro_bb = ['N', 'O', 'C', 'CA']
         pro_sc = {'A': ['CB'], 'C': ['CB', 'SG'], 'D': ['CB', 'CG', 'OD1', 'OD2'], 'E': ['CB', 'CG', 'CD', 'OE1',
                   'OE2'], 'F': ['CB', 'CG', 'CD1', 'CD2', 'CE1', 'CE2', 'CZ'], 'G': [], 'H': ['CB', 'CG', 'ND1', 'CE1',
@@ -251,7 +250,45 @@ class Pdb:  # TODO optionally save as gro? & think of trajectories
                                               atom_instance.serial, atom_instance.atomname))
             return 1
         return 0
-        
+
+    def print_mols(self):
+        chains = list({a.chain.strip() for a in self.atoms})
+        if not chains:
+            print("No chains in the molecule, run Pdb.add_chains() first")
+        def identify(atom):
+            first = atom.resname
+            if first in Pdb.prot_map.keys() or (first[1:] in Pdb.prot_map.keys() and first[0] in 'NC'):
+                return 'Protein'
+            elif first in Pdb.nucl_map.keys():
+                return 'Nucleic'
+            else:
+                return atom.resname
+        mol_list = []
+        mol = identify(self.atoms[0])
+        chain = self.atoms[0].chain
+        res = self.atoms[0].resnum
+        for a in self.atoms:
+            if mol in ['Protein', 'Nucleic']:
+                if a.chain != chain:
+                    mol_list.append([mol, 1])
+                    chain = a.chain
+                    mol = identify(a)
+            else:
+                if a.resnum != res:
+                    if len(mol) > 1 and mol == mol_list[-1][0]:
+                        mol_list[-1][1] += 1
+                    else:
+                        mol_list.append([mol, 1])
+                    mol = identify(a)
+                    chain = a.chain
+            res = a.resnum
+        if len(mol) > 1 and mol == mol_list[-1][0] and mol not in ['Protein', 'Nucleic']:
+            mol_list[-1][1] += 1
+        else:
+            mol_list.append([mol, 1])
+        for i in mol_list:
+            print(f'{i[0]} {i[1]}')
+
     def _remove_altloc(self):
         """
         We only keep one of the alternative locations in case
@@ -311,12 +348,12 @@ class Pdb:  # TODO optionally save as gro? & think of trajectories
                 new_atoms.append(a)
         self.atoms = new_atoms
     
-    def renumber_atoms(self):
-        for n, atom in enumerate(self.atoms, 1):
+    def renumber_atoms(self, offset=1):
+        for n, atom in enumerate(self.atoms, offset):
             atom.serial = n
     
-    def renumber_residues(self):
-        count = 1
+    def renumber_residues(self, offset=1):
+        count = offset
         for n in range(len(self.atoms)):
             temp = count
             try:
@@ -466,12 +503,12 @@ class Pdb:  # TODO optionally save as gro? & think of trajectories
         return at2.x - at1.x, at2.y - at1.y, at2.z - at1.z
         
     @staticmethod
-    def _parse_contents(contents):
+    def _parse_contents(contents, qt):
         atoms, remarks, conect = [], [], {}
         box = [7.5, 7.5, 7.5, 90, 90, 90]  # generic default, will be overwritten if present
         for line in contents:
             if line.startswith('ATOM') or line.startswith('HETATM'):
-                atoms.append(Atom(line))
+                atoms.append(Atom(line, qt))
             elif line.startswith("CRYST1"):
                 box = [float(line[6+9*a:6+9*(a+1)]) for a in range(3)] + \
                       [float(line[33+7*a:33+7*(a+1)]) for a in range(3)]
@@ -558,6 +595,24 @@ class Pdb:  # TODO optionally save as gro? & think of trajectories
         f = -1 if nv == 3 else 1
         return f*sum(v[0] for v in vecs)/nv, f*sum(v[1] for v in vecs)/nv, f*sum(v[2] for v in vecs)/nv
 
+    def add_vs2(self, resid, name1, name2, vsname='V1', fraction=0.5):
+        indx = self.get_atoms(f"resid {resid}")[-1].serial + 1
+        a1 = self.get_atom(f"resid {resid} and name {name1}")
+        a2 = self.get_atom(f"resid {resid} and name {name2}")
+        dist = self._atoms_dist_pbc(a1, a2)
+        self.insert_atom(indx, self.get_atom(f"resid {resid} and name {name1}"),
+                         atomsel=f"resid {resid} and name {vsname}",
+                         hooksel=f"resid {resid} and name {name1}",
+                         bondlength=dist * fraction, p1_sel=f"resid {resid} and name {name1}",
+                         p2_sel=f"resid {resid} and name {name2}", atomname=vsname)
+
+    def interatomic_dist(self, resid1=1, resid2=2):
+        dists = []
+        for atom1 in self.get_atoms(f"resid {resid1}"):
+            for atom2 in self.get_atoms(f"resid {resid2}"):
+                dists.append(self._atoms_dist(atom1, atom2))
+        return dists
+
     @staticmethod
     def _parse_contents_gro(contents):
         import math
@@ -694,9 +749,12 @@ class Pdb:  # TODO optionally save as gro? & think of trajectories
             
 
 class Atom:
-    def __init__(self, line):
+    def __init__(self, line, qt=False):
         self.label = line[:6].strip()
-        self.serial = int(line[6:11].strip())
+        try:
+            self.serial = int(line[6:11].strip())
+        except ValueError:
+            self.serial = int(line[6:11].strip(), 16)
         self.atomname = line[12:16].strip()
         self.altloc = line[16:17]
         self.resname = line[17:21].strip()
@@ -704,8 +762,16 @@ class Atom:
         self.resnum = int(line[22:26].strip())
         self.insert = line[26:27]
         self.x, self.y, self.z = [float(line[30+8*a:30+8*(a+1)]) for a in range(3)]
-        self.occ = float(line[54:60].strip())
-        self.beta = float(line[60:66].strip())
+        if not qt:
+            self.occ = float(line[54:60].strip())
+            self.beta = float(line[60:66].strip())
+            self.q = 0.0
+            self.type = 'X'
+        else:
+            self.occ = 1.0
+            self.beta = 0.0
+            self.q = float(line[54:62].strip())
+            self.type = str(line[62:].strip())
         try:
             self.element = line[76:78].strip()
         except IndexError:
