@@ -226,16 +226,16 @@ class SubsectionBonded(Subsection):
                                 pass
         return entries
 
-    def find_missing_ff_params(self, fix_by_analogy=False, fix_B_from_A=False, fix_A_from_B=False):
+    def find_missing_ff_params(self, fix_by_analogy=False, fix_B_from_A=False, fix_A_from_B=False, once=False):
         matchings = {'bonds': 'bondtypes', 'angles': 'angletypes', 'dihedrals': 'dihedraltypes',
                      'impropers': 'dihedraltypes'}
         subsect_params = [sub for sub in self.section.top.parameters.subsections if
                           sub.header == matchings[self.header]]
         for entry in self.entries:
             if isinstance(entry, gml.EntryBonded):
-                self._find_missing_ff_params(entry, subsect_params, fix_by_analogy, fix_B_from_A, fix_A_from_B)
+                self._find_missing_ff_params(entry, subsect_params, fix_by_analogy, fix_B_from_A, fix_A_from_B, once)
 
-    def _find_missing_ff_params(self, entry, subsect_params, fix_by_analogy, fix_B_from_A, fix_A_from_B):
+    def _find_missing_ff_params(self, entry, subsect_params, fix_by_analogy, fix_B_from_A, fix_A_from_B, once):
         if (entry.params_state_a and entry.params_state_b) or (entry.params_state_a and not entry.types_state_b):
             return
         int_type = entry.interaction_type
@@ -246,8 +246,11 @@ class SubsectionBonded(Subsection):
                 if parm_entry.match(entry.types_state_a, int_type):
                     found_a = True
         if not found_a and not entry.params_state_a:
-            print(f'Couldn\'t find params for interaction type {entry.subsection.header} {int_type}, '
-                  f'atom types {entry.types_state_a}, atom numbers {entry.atom_numbers}')
+            if not once or entry.types_state_a not in self.section.printed:
+                print(f'Couldn\'t find params for interaction type {entry.subsection.header} {int_type}, '
+                      f'atom types {entry.types_state_a}, atom numbers {entry.atom_numbers}')
+            if once and entry.types_state_a not in self.section.printed:
+                self.section.printed.append(entry.types_state_a)
             if fix_by_analogy:
                 candid = self._fix_by_analogy(fix_by_analogy, entry.types_state_a, subsect_params, int_type)
                 if candid:
@@ -258,8 +261,11 @@ class SubsectionBonded(Subsection):
                     if parm_entry.match(entry.types_state_b, int_type):
                         found_b = True
             if not found_b and not entry.params_state_b:
-                print(f'Couldn\'t find params for interaction type {entry.subsection.header} {int_type}, '
-                      f'atom types {entry.types_state_b}, atom numbers {entry.atom_numbers}')
+                if not once or entry.types_state_b not in self.section.printed:
+                    print(f'Couldn\'t find params for interaction type {entry.subsection.header} {int_type}, '
+                          f'atom types {entry.types_state_b}, atom numbers {entry.atom_numbers}')
+                if once and entry.types_state_b not in self.section.printed:
+                    self.section.printed.append(entry.types_state_b)
                 if fix_by_analogy:
                     candid = self._fix_by_analogy(fix_by_analogy, entry.types_state_b, subsect_params, int_type)
                     if candid:
@@ -277,7 +283,7 @@ class SubsectionBonded(Subsection):
                     entry.params_state_a = candid
 
     @staticmethod
-    def _fix_by_analogy(subst, typelist, subsect_params, int_type, other_typelist = None):
+    def _fix_by_analogy(subst, typelist, subsect_params, int_type, other_typelist=None):
         new_params = []
         from_wildtype = None
         if other_typelist is None:
@@ -466,6 +472,19 @@ class SubsectionParam(Subsection):
                          for n, entry in enumerate(self.entries)}
         self.entries.sort(key=self._sorting_fn)
 
+    def find_used_ff_params(self):
+        used_parm_entries = []
+        used_atomtypes_a = {a.type for mol in self.section.top.molecules for a in mol.atoms
+                            if isinstance(a, gml.EntryAtom)}
+        used_atomtypes_b = {a.type_b for mol in self.section.top.molecules for a in mol.atoms
+                            if isinstance(a, gml.EntryAtom) and a.type_b is not None}
+        used_atomtypes = used_atomtypes_a.union(used_atomtypes_b)
+        for entry in self.entries:
+            if isinstance(entry, gml.EntryParam):
+                if all(tp in used_atomtypes for tp in entry.types):
+                    used_parm_entries.append(entry.identifier)
+        return used_parm_entries
+
     def _sorting_fn(self, entry):
         """
         Comments should go first, then we sort based on first, second,
@@ -496,9 +515,12 @@ class SubsectionParam(Subsection):
                     return entry.content[npar]
         return '0'
 
-    def get_opt_dih(self):
+    def get_opt_dih(self, types=False):
         dopts = [entry for entry in self.entries if isinstance(entry, gml.EntryParam) and 'DIHOPT' in entry.comment]
-        return [e.params[x] for e in dopts for x in [0, 1]]
+        if not types:
+            return [e.params[x] for e in dopts for x in [0, 1]]
+        else:
+            return [(*e.types, e.params[2]) for e in dopts]
 
     def get_opt_dih_indices(self):
         dopts = [entry for entry in self.entries if isinstance(entry, gml.EntryParam) and 'DIHOPT' in entry.comment]
@@ -567,35 +589,7 @@ class SubsectionAtom(Subsection):
     def __init__(self, content, section):
         super().__init__(content, section)
         self.fstring = "{:>6}{:>11}{:>7}{:>7}{:>7}{:>7}{:>11}{:>11}   ; " + '\n'
-        self.nat, self.charge = None, None
-        self.calc_properties()
         self.name_to_num, self.num_to_name, self.num_to_type, self.num_to_type_b = None, None, None, None
-    
-    def calc_properties(self):
-        """
-        Recalculates molecule properties: number of atoms and total charge
-        :return: None
-        """
-        self.nat = self.section.natoms = self._calc_nat()
-        self.charge = self.section.charge = self._calc_charge()
-        
-    def _calc_charge(self):
-        """
-        Calculates total charge of the molecule
-        :return: float, total charge
-        """
-        total_charge = 0
-        for entry in self.entries:
-            if isinstance(entry, gml.EntryAtom):
-                total_charge += entry.charge
-        return total_charge
-    
-    def _calc_nat(self):
-        """
-        Counts atoms in molecules
-        :return:
-        """
-        return len([e for e in self.entries if isinstance(e, gml.EntryAtom)])
 
     def get_dicts(self, force_update=False):
         """
